#!/usr/local/bin/perl5
########################################################################
# Name:    checksum_update.pl - 9.2 Checksums update tool
#
# Support: 
#
# Purpose: This program  updates the SAS checksum files used in
#          SAS Installation Qualification.  The program reads a file
#          containing update directives and applies those directives
#          to the production checksum files.  A backup of the production
#          checksum files is performed as part of this operation. 
#
# History:
#          25Apr2008 saskww   Initial coding
#          18nov2008 saskww   removed dependency on bash by adding
#                             -vmi option; corrected naming of
#                             display_usage subroutine.
#
# Copyright (c) 2008 by SAS Institute Inc., Cary NC
########################################################################

use strict;
use File::Path;
use Getopt::Long;
use File::Basename;
use Time::Local;
use File::Spec;


#--------------------------------------------------------------------
# Display the usage notes.
#--------------------------------------------------------------------
sub display_usage
{
   print <<______printEnd;

NAME
	checksum_update.pl - update checksums as part of hotfix installation

SYNOPSIS
        checksum_update.pl -hotfix <HotFixID> -sasroot <SASROOTPath>

        checksum_update.pl -hotfix <HotFixID> -sasroot <SASROOTPath> 
                           -filelist <update directives file> 

DESCRIPTION
	This program  updates the SAS checksum files used in SAS
	Installation Qualification.  The program reads a file containing
	update directives and applies those directives to the production
	checksum files.  A backup of the production checksum files is
	performed as part of this operation. 

    Options
	The check_update.pl command recognizes the following options.  Note
	that options can be abbreviated to the least common string
	(i.e. -ho for hotfix, -he for help, -s for sasroot).

           -hotfix hotfixid	The hotfixid.  If the filelist option is 
				not specified, this will be used to derive
				the name of the directives file:
				\$SASROOT/9.2/install/admin/hotfix/<HFID>.UPDATES

           -sasroot path	The SASROOT path of a current, existing install.
				This is needed in order to locate an up-to-date
				version of the original install checksum files.
				This should be stated down to the version
				(i.e. 9.2).

           -filelist file       This will override the default hotfix file
				location.

           -logfile file	This will override the default log file
				location.

	   -check		This will cause the program parameters to be
				displayed.

           -verbose		This will cause information verbose outout to
				be displayed.

           -test		This will prevent the actual update to the 
				production checksum files.  A message will be
				displayed showing the operations that would
				be performed when not testing.

RETURN VALUE
        The check_update.pl command returns 0 if updates are successful.	

EXAMPLES

	The following example uses the default update directives file:

		checksum_update.pl  -ho fb9ba04 -s /DISK/SASFoundation/9.2


FILE LOCATIONS:
	Checksum file backups:
		\$SASROOT/9.2/install/admin/backups/hotfixid
			install-admin-checksums-BASE_BAPR00EN.chksum.04222008.124406 

	Update Directives file (default):
		\$SASROOT/9.2/install/admin/hotfix
			F9BA02R6.UPDATES
	Script
		\$SASROOT/9.2/install/admin/hotfix/script
			checksum_update.pl

	Log   
		\$SASROOT/9.2/install/admin/hotfix
			f9ba02r6.log.04222008.124406
______printEnd

   exit 0;
}

#--------------------------------------------------------------------------
# Log messags to the logfile.  If the ReturnCode is not equal to 0,
# exit the program using the ReturnCode as the exit code.
#--------------------------------------------------------------------------
sub LogIt
{
   my $LogText = shift;
   my $ReturnCode = shift;
   my $verbose = shift;

   my $rctext="(rc=$ReturnCode)" if $ReturnCode ne "0" && $ReturnCode ne "";

   print LOGIT "$LogText $rctext\n";
   print "LOGGING:  $LogText rc=$ReturnCode\n" if $verbose; 

   exit ($ReturnCode) if $ReturnCode ne "0" && $ReturnCode ne "";
}

#==========================================================================
# MAIN PROGRAM
#==========================================================================

&print_usage if $#ARGV < 0;

#--------------------------------------------------------------------------
# Process the options.  GetOptions will allow parameters to be entered
# in either long or short names, and will allow any case.
#--------------------------------------------------------------------------
GetOptions(
   "check",       \my $check,
   "filelist=s",  \my $filelist,
   "hotfixid=s",  \my $hotfixID,
   "help|?",      \&print_usage,
   "logfile=s",   \my $logfile,
   "sasroot=s",   \my $SASROOT,
   "test",        \my $test,
   "verbose",     \my $verbose,
   "vmi",         \our $vmi,
);

#--------------------------------------------------------------------
# If VMI, add additional use...
#--------------------------------------------------------------------
if ( $vmi )
{
   print "On VMI, so use VMS::Filespec.\n" if $verbose;
   use VMS::Filespec ;
}

#--------------------------------------------------------------------------
# Make sure required options are there
# On VMI, there is a requirement to run a SAS program that sets up
# SASROOT, so, this is not required on that platform, and, in fact,
# will not be used.
#--------------------------------------------------------------------------
&display_usage if !$hotfixID;

if ( !$vmi )
{
   &display_usage if !$SASROOT;

   #--------------------------------------------------------------------------
   # Make sure SASROOT exists.  Die returns 2. 
   #--------------------------------------------------------------------------
   &LogIt ( "SASROOT $SASROOT does not exist.",10, $verbose ) if !-d $SASROOT;
}

#--------------------------------------------------------------------------
# If VMI, set the variables appropariate
#--------------------------------------------------------------------------
my $SASWORK="";
if ( $vmi )
{
   $SASWORK= $ENV{"SAS\$WORKROOT"};
   print "SASWORK is $SASWORK\n" if $verbose;
   $SASWORK = VMS::Filespec::unixify($SASWORK);
   # unixify is stupid.  It resolves:
   # any "9^.2" to 9^/2
   # change that to 9.2
   $SASWORK =~ s/\^\//./;
   #$SASWORK =~ s/\$/\\\$/;
   print "after unixify, SASWORK is $SASWORK\n" if $verbose;

   $SASROOT= $ENV{"SAS\$ROOT"};
   $SASROOT = VMS::Filespec::unixify($SASROOT);
   # unixify is stupid.  It resolves:
   # SASFOUNDATION.9^.2. to SASFoundation/9^/2/
   # Change it to SASFoundation/9^.2/
   $SASROOT =~ s/\^\//./;
   #$SASROOT =~ s/\$/\\\$/g;
   print "after unixify SASROOT is $SASROOT\n" if $verbose;

   if ( $filelist )
   {
      $filelist = VMS::Filespec::unixify($filelist);
      # unixify is stupid.  It resolves:
      # any "9^.2" to 9^/2
      # change that to 9.2
      $filelist =~ s/\^\//./;
      #$filelist =~ s/\$/\\\$/;
      print "$filelist is $filelist\n" if $verbose;
   }

   if ( $logfile )
   {
      print "Raw logfile name is $logfile\n" if $verbose;
      my $dirname=dirname($logfile);
      my $filename=basename($logfile);
      $logfile = VMS::Filespec::unixify($dirname);
      # unixify is stupid.  It resolves:
      # any "9^.2" to 9^/2
      # change that to 9.2
      $logfile =~ s/\^\//./;
      #$logfile =~ s/\$/\\\$/;
      print "logfile  (dir) is $logfile\n" if $verbose;
      print "logfile is $logfile\n" if $verbose;
      $logfile .= "$filename";
      print "logfile is $logfile\n" if $verbose;
   }
}

#--------------------------------------------------------------------------
# DateTime used in backup file names and log dir name.
#--------------------------------------------------------------------------
my $ToolVer="1.0";

my $ctime=time();
my $ltime=localtime($ctime);

#--------------------------------------------------------------------------
# Set the logfile, and attempt to open it.  If opening it fails, issue
# an error and exit.
#--------------------------------------------------------------------------
$logfile="${SASROOT}/install/admin/hotfix/${hotfixID}_${ctime}.log" if !$logfile && !$vmi;
$logfile="${SASROOT}install/admin/hotfix/${hotfixID}_${ctime}.log"  if !$logfile && $vmi;

# Convert LOGFILE to VMS syntax if on VMI
if ( $vmi )
{
   ( my $vlogfile=VMS::Filespec::vmsify($logfile) ) =~ s/9_2/9^.2/;
   open (LOGIT,">$vlogfile") || die ( "Error opening log file $vlogfile: $! \n");
}
else
{
   open (LOGIT,">$logfile") || die ( "Error opening log file $logfile: $! \n");
}

print "Successfully opened logfile $logfile.\n" if $verbose;
&LogIt ("UNIX Checksum Update Tool Version $ToolVer",0);
&LogIt ("Log generated: $ltime [$ctime]",0);


#--------------------------------------------------------------------------
# Display the operational parameters if check...
#--------------------------------------------------------------------------
if ($check)
{
   print <<______printEnd;

hotfixID:         $hotfixID
filelist:         $filelist
SASRoot:          $SASROOT
______printEnd

   print "verbose:          TRUE\n"  if $verbose;
   print "verbose:          FALSE\n" if !$verbose;

   print "OS (UNIX or VMI): VMI\n"   if $vmi;
   print "OS (UNIX or VMI): UNIX\n"  if !$vmi;

   exit 0;
}

#--------------------------------------------------------------------------
# If there was no filelist entered on the command line, use the
# default based on the hotfixid.  Die returns 2.
#--------------------------------------------------------------------------
my $uchotfixID=uc($hotfixID);
if ( $vmi )
{
   $filelist="${SASROOT}install/admin/hotfix/${uchotfixID}.UPDATES"  if !$filelist;
   ( $filelist = VMS::Filespec::vmsify($filelist) ) =~ s/9_2/9^.2/;
   print "Filelist is $filelist\n" if $verbose;
}
else
{
   $filelist="${SASROOT}/install/admin/hotfix/${uchotfixID}.UPDATES" if !$filelist;
   print "Filelist is $filelist\n" if $verbose;
}
&LogIt ("UPDATES file $filelist does not exist.",20, $verbose ) if !-f $filelist;

#--------------------------------------------------------------------------
# Set the CksumBkDir.  This should exist at this point in the process...
# Die returns 2.
#--------------------------------------------------------------------------
my $CksumBkDir;
my $SysCmd;
if ( $vmi )
{
   $CksumBkDir="${SASROOT}install/admin/backups/${hotfixID}/";
   ( $CksumBkDir = VMS::Filespec::vmsify($CksumBkDir) ) =~ s/9_2/9^.2/;
   print "Checksum backup directory is $CksumBkDir\n" if $verbose;
   $SysCmd="create/dir $CksumBkDir";
}
else
{
   $CksumBkDir="${SASROOT}/install/admin/backups/${hotfixID}";
   print "Checksum backup directory is $CksumBkDir\n" if $verbose;
   $SysCmd="mkdir $CksumBkDir";
}

if ( !-d $CksumBkDir )
{
   if ( $test )
   {
      &LogIt ("TEST:  $SysCmd",0, $verbose);
   }
   else
   {
      &LogIt ("Running 01:  $SysCmd",0, $verbose);
      my $mkdirrc=system( "$SysCmd" );
      &LogIt ("Error creating $CksumBkDir rc=$mkdirrc.",30, $verbose) if $mkdirrc != 0;
   }
}

#--------------------------------------------------------------------------
# Read the file containing the updates.  Each line is formated as follows:
#
# -- Checksum file ---:-file-:-path-:------------checksum------------
# BASE_BAPR00EN.chksum:sasxml:sasexe:044ac754b95e00c5c5007a539e381196
#
# While reading the file, push the directives to the @Directives array.
# Also, push the Checksum file to the @ChkSumFiles array.  This will later
# be sorted and uniqufied.
#
# Die returns 2.
#--------------------------------------------------------------------------
my @Directives = ();
my @ChkSumFiles = ();
print "About to open checksumfile $filelist.\n";
open (CHKSUMFILE,"<$filelist") || &LogIt ("Error opening $filelist.",40, $verbose );
&LogIt ("\nReading update directives file: $filelist",0, $verbose);
while (<CHKSUMFILE>)
{
   chop;
   print "This checksum file is $_\n" if $verbose;
   (my $csfile, my $file, my $path, my $checksum) = split (":", $_ );
   push (@Directives, $_);
   push (@ChkSumFiles, $csfile);
}

# Sort and uniquify the ChkSumFiles
sort @ChkSumFiles;
my %seen = ();
@ChkSumFiles = grep { ! $seen{$_} ++ } @ChkSumFiles;

#--------------------------------------------------------------------------
# FOREACH Checksum file in the array:
#
# - open the production file for read
# - read production file lists into arrays:
#   - topjunk
#   - file lines
#   - bottom junk
# - Create a temporary file
# - open the temporary file for write
# - if there are updates
#   - backup the production file
#   - copy the temp file over the production file
# - remove the temporary file
#--------------------------------------------------------------------------
my @SuccessUpdateFiles=();
my $FileUpdated=0;
my $FileNum=0;
foreach my $checksumfile (@ChkSumFiles)
{
   $FileNum+=1;
   &LogIt ("\nProcessing updates for $checksumfile",0, $verbose);

   #---
   # Load the directives for this line into a temporary array
   #---
   my @TmpDirectives=grep /$checksumfile/, @Directives;


   #---
   # - open the production file for read.  Exit on failure.  Die returns 2;
   # - read production file lists into arrays:
   #   - topjunk
   #   - file lines
   #   - bottom junk
   #---
   my $ProdChksumFile;
   if ( $vmi )
   {
      $ProdChksumFile="${SASROOT}/install/admin/checksums/$checksumfile";
      ( $ProdChksumFile = VMS::Filespec::vmsify($ProdChksumFile) ) =~ s/9_2/9^.2/;
      print "Production checksum file is $ProdChksumFile\n" if $verbose;
   }
   else
   {
      $ProdChksumFile="${SASROOT}install/admin/checksums/$checksumfile";
      print "Production checksum file is $ProdChksumFile\n" if $verbose;
   }

   open ( INCSF, "<$ProdChksumFile" ) || &LogIt ("Error opening checksum file $ProdChksumFile for read.",50, $verbose );
   &LogIt ("\nOpened $ProdChksumFile.",0, $verbose);
   my @TopJunk, my @FileLines, my @BottomJunk;
   while (<INCSF>)
   {
      chop;

      # If it is a <file line, push it to the FileLines array
      push ( @FileLines, $_ ) if $_ =~ /<file /;
      next if $_ =~ /<file /;

      # If it is a closing ( </ ), push it to the BottomJunk array
      push ( @BottomJunk, $_ ) if $_ =~ /<\//;
      next if $_ =~ /<\//;

      # Anything else must be topi stuff, push it to the TopJunk array
      push ( @TopJunk, $_ );
   }
   close INCSF;

   #---
   # Create temporary chksum file.  The name is based on the checksum
   # file we are processing.  Open it for write, and load the top
   # junk into the file.  If there is a failure opening the temp
   # file, exit.  Die returns 2.
   #---
   my $TmpChksumFile;
   if ( $vmi )
   {
      # Need "_$$" on VMI...  otherwise, there are two "." in the
      # filename, and apparently, that's a nono.
      $TmpChksumFile="${CksumBkDir}${checksumfile}_$$";
      print "Temporary checksum file is $TmpChksumFile\n" if $verbose;
      ( $TmpChksumFile = VMS::Filespec::vmsify($TmpChksumFile) ) =~ s/9_2/9^.2/;
      print "Temporary checksum file is $TmpChksumFile\n" if $verbose;
   }
   else
   {
      $TmpChksumFile="/tmp/$checksumfile.$$";
      print "Temporary checksum file is $TmpChksumFile\n" if $verbose;
   }

   # Open temporary checksum file for write
   open ( TMPFILE, ">$TmpChksumFile" ) || &LogIt ("Error opening $TmpChksumFile for write.\n",60, $verbose);

   # Load the topjunk in the temporary file
   foreach my $ThisJunkLine (@TopJunk)
   { print TMPFILE "$ThisJunkLine\n"; }

   #---
   # Start with fileupdated to false...  we will know if we need to
   # write the tmp file is the file is updated.
   #---
   $FileUpdated=0;

   #---
   # Now, for each line in the file, let's munge the information into the
   # format of the update directives.  Then, we will search for that
   # string in the @TmpDirectives array.  There should be:
   #  -- NO matches, meaning the information does not need to be updated
   #  -- ONE match, meaning the information does need to be updated
   # If there are more than or match... UHOH.  How could that happen???
   # Something funky is going on with this hotfix/input file...
   #---
   foreach my $ThisFileLine ( @FileLines )
   {
      #---
      # The format of ThisFileLine is:
      #     <file filename="saszfctl" relativepath="dbcs/sasexe" rootflag="" checksum="7bff717e8b48ec7f1014cd2ebc0c1cf2"/>
      #
      # We're going to load the file into workline, then, parse it
      # apart to get the values needed to create the match line.
      #---
      
      ( my $workline = $ThisFileLine ) =~ s/^\s+//;
      my @fields = split(" ", $workline);
      my $fkeyword, my $fvalue, my $pkeyword, my $pvalue, my $ckeyword, my $cvalue;
      foreach my $thisfield (@fields)
      {
         ( ($fkeyword, $fvalue ) = split ('"', $thisfield) ) if ( $thisfield =~ /filename=/);
         ( ($pkeyword, $pvalue ) = split ('"', $thisfield) ) if ( $thisfield =~ /relativepath=/);
         ( ($ckeyword, $cvalue ) = split ('"', $thisfield) ) if ( $thisfield =~ /checksum=/);
      }
      
      #---
      # Now that we have the information, construct the match line similar to:
      #  BASE_BAPR00EN.chksum:fsp.sas7bcat:sashelp:c52aa466c24c13c6e82890804579633c
      #---
      my $matchString="$checksumfile:$fvalue:$pvalue";

      #---
      # Grep for the matchString in the TmpDirectives array.  Any matches
      # will be in the matches array...
      #---
      my @matches = grep /$matchString/, @TmpDirectives;
      my $NumMatches=scalar(@matches);

      #---
      # Techincally, there should be only one match...  For now, we'll
      # message and continue.  May really need to abort...
      #---
      &LogIt ("UH-OH. More than one match.  This *REALLY* should not happen!",0, $verbose) if $NumMatches > 1;

      #---
      # If we found a match, let's update the checksum information in 
      # workline and write it to the temporary checksum file
      #---
      if ( $NumMatches == 1 )
      {
         $FileUpdated=1;
         &LogIt ("Made $NumMatches match on: $matchString",0, $verbose) if $verbose;

         #---
         # Read the Directive that matched from matches, then parse the
         # line to get the new checksum.  Then, substitute the  original
         # checksum (cvalue derived above) with the new checksum  in the
         # workline and write it out to the temporary file.
         #---
         my $MatchedDirective = @matches[0];
         ( my $junkcfile, my $junksfile, my $junkpath, my $newcsum ) = split (":", $MatchedDirective );
         $workline =~ s/$cvalue/$newcsum/;
         print TMPFILE "    $workline\n";
         &LogIt ("\n\tReplacing:  $ThisFileLine\n\twith:           $workline",0);

         # Remove the matches from the directives array
         @TmpDirectives=grep !/$matchString/, @TmpDirectives;
      }
      
      #---
      # If no matches, just print the original line to the file
      #---
      if ( $NumMatches == 0 )
      {
         print TMPFILE "$ThisFileLine\n";
      }
   }

   #--- 
   # If there are still directives for this file, need to load them in.
   #
   # Convert:
   #     BASE_BAPR00EN.chksum:sasnewfi:dbcs/sasexe:600b6aee4187ec80df2e7a323f03b172
   # to:
   #     <file filename="saszfctl" relativepath="dbcs/sasexe" rootflag="" checksum="7bff717e8b48ec7f1014cd2ebc0c1cf2"/>
   #---
   foreach my $ThisAddition (@TmpDirectives )
   {
      $FileUpdated=1;
      ( my $junk, my $filename, my $filepath, my $checksum ) = split (":", $ThisAddition );
      my $addline="    <file filename=\"$filename\" relativepath=\"$filepath\" rootflag=\"\" checksum=\"$checksum\"/>";
      print TMPFILE "$addline\n";
      &LogIt ("\n\tAdding:     $addline",0);
   }
   
   #---
   # Load the bottomjunk in the temporary file
   #---
   foreach my $ThisJunkLine (@BottomJunk)
   { print TMPFILE "$ThisJunkLine\n" ; }

   close TMPFILE;
   &LogIt ("Created temporary checksum file:  $TmpChksumFile",0, $verbose);
   
   #---
   # If the file needs to be updated, we will need to backup the checksum
   # file.  If the backup was successful, then copy the temporary
   # checksum file over the original, and, once that is successful, 
   # remove the temporary file.
   #---
   if ( $FileUpdated )
   {

      #---
      # Copy the original checksum file to the backup directory as, for example:
      #
      # install-admin-checksums-BASE_BAPR00EN.chksum.04222008.124406
      #---
      my $OrgChksFile;
      my $CksumBkFile;
      if ( $vmi )
      {
         $OrgChksFile="${SASROOT}install/admin/checksums/${checksumfile}";
         $CksumBkFile="${CksumBkDir}install-admin-checksums-${checksumfile}_${ctime}",
         ( $OrgChksFile = VMS::Filespec::vmsify($OrgChksFile) ) =~ s/9_2/9^.2/;
         ( $CksumBkFile = VMS::Filespec::vmsify($CksumBkFile) ) =~ s/9_2/9^.2/;
         print "Original checksum file is $OrgChksFile\n" if $verbose;
         print "Backup checksum file is $CksumBkFile\n" if $verbose;
      }
      else
      {
         $OrgChksFile="${SASROOT}/install/admin/checksums/${checksumfile}";
         $CksumBkFile="${CksumBkDir}/install-admin-checksums-${checksumfile}.${ctime}";
      }

      # Set CksumBkFile 
      LogIt ("CksumBkFile will be $CksumBkFile.",0, $verbose);

      my $SysCmd="cp -p $OrgChksFile $CksumBkFile" if !$vmi;
      my $SysCmd="copy  $OrgChksFile $CksumBkFile" if $vmi;
      if ( $test )
      {
         &LogIt ("TEST: $SysCmd",0, $verbose);
      }
      else
      {
         &LogIt ("Running 02:  $SysCmd",0, $verbose);
         my $bkrc=system( "$SysCmd" );
         &LogIt ("Error copying $OrgChksFile to $CksumBkFile rc=$bkrc.  Continuing.",65, $verbose) if $bkrc != 0;
         &LogIt ("\nBacked up $OrgChksFile to $CksumBkFile.",0, $verbose) if $bkrc == 0;
      }
    
      #---
      # Copy the temporary file over the original file.  Exit on error.  Die
      # returns 2.
      #---
      my $SysCmd="cp -p $TmpChksumFile $OrgChksFile" if !$vmi;
      my $SysCmd="copy  $TmpChksumFile $OrgChksFile" if $vmi;
      if ( $test )
      {
         &LogIt ("TEST: $SysCmd",0, $verbose);
      }
      else
      {
         &LogIt ("Running 03:  $SysCmd",0, $verbose);
         my $copyrc=system( "$SysCmd" );
         &LogIt ("Error copying $TmpChksumFile to $OrgChksFile rc=$copyrc.",70, $verbose) if $copyrc != 0;
         &LogIt ("Copied $TmpChksumFile to $OrgChksFile.",0, $verbose) if $copyrc == 0;
         push (@SuccessUpdateFiles,$OrgChksFile) if $copyrc == 0;   
      }

      #---
      # Remove the temporary file
      #---
      my $SysCmd="rm     $TmpChksumFile" if !$vmi;
      my $SysCmd="delete ${TmpChksumFile}*.*" if $vmi;
      if ( $test )
      {
         &LogIt ("TEST:  $SysCmd",0, $verbose);
      }
      else
      {
         &LogIt ("Running 04:  $SysCmd",0, $verbose);
         my $rmrc=system( "$SysCmd" );
         &LogIt ("Error removing $TmpChksumFile.",0, $verbose) if $rmrc != 0;
      }
   }
   else
   {
      #---
      # If there are no updates, remove the temporary file.  I really would
      # not expect this to happen, but we need to clean up just in case...
      #---
      my $SysCmd="rm $TmpChksumFile";
      if ( $test )
      {
         &LogIt ("TEST:  $SysCmd",0, $verbose);
      }
      else
      {
         &LogIt ("Running 05:  $SysCmd",0, $verbose);
         my $rmrc=system( "$SysCmd" );
         &LogIt ("Error removing $TmpChksumFile rc=$rmrc.",0, $verbose) if $rmrc != 0;
      }
   }
}

&LogIt ("\nThe following checksum files were updated:",0,$verbose);
foreach my $checksumfile (@SuccessUpdateFiles)
{
   #my $logtxt="${SASROOT}/install/admin/checksums/$checksumfile" if !$vmi;
   #my $logtxt="${SASROOT}install/admin/checksums/$checksumfile"  if $vmi;
   &LogIt ("\t${checksumfile}",0,$verbose);
}

close LOGIT;
exit;
